﻿using BrainPad;

namespace Invader
{
    class Sprite
    {
        Image Picture;
        public int X, Y;
        public int Width;
        public int Height;

        public Sprite(Image p, int x, int y)
        {
            Picture = p;
            X = x;
            Y = y;
            Width = p.Width;
            Height = p.Height;
        }

        public void Clear()
        {
            for (int i = this.X; i < this.X + this.Width; i++)
            {
                for (int j = this.Y; j < this.Y + this.Height; j++)
                {
                    Display.Point(i, j, 0);
                }
            }
        }

        public void Draw()
        {
            Display.Image(Picture, X, Y);
        }
    }

    class Program
    {
        static int score = 0;
        static int ShipNumber = 3;
        static int MonsterDelta = 5;

        static Button BtnA = new Button("A", 0);
        static Button BtnB = new Button("B", 0);
        static Sound speaker = new Sound("buzzer", 0.05, 100);

        static Image ShipImage = Display.CreateImage(8, 5, new byte[] {
            0,0,0,1,1,0,0,0,
            0,0,0,1,1,0,0,0,
            0,1,1,1,1,1,1,0,
            1,1,1,1,1,1,1,1,
            1,0,0,0,0,0,0,1,
            }, 1, 1, 1);

        static Image BulletImage = Display.CreateImage(2, 2, new byte[] {
            1,1,
            1,1,
            }, 1, 1, 1);

        static Image MonsterImage = Display.CreateImage(8, 8, new byte[] {
            0,0,0,1,1,0,0,0,
            0,0,1,1,1,1,0,0,
            0,1,1,1,1,1,1,0,
            1,1,0,1,1,0,1,1,
            1,1,1,1,1,1,1,1,
            0,0,1,0,0,1,0,0,
            0,1,0,1,1,0,1,0,
            1,0,1,0,0,1,0,1,
            }, 1, 1, 1);

        static Image MonsterReverseImage = Display.CreateImage(8, 8, new byte[] {
            1,1,1,0,0,1,1,1,
            1,1,0,0,0,0,1,1,
            1,0,0,0,0,0,0,1,
            0,0,1,0,0,1,0,0,
            0,0,0,0,0,0,0,0,
            1,1,0,1,1,0,1,1,
            1,0,1,0,0,1,0,1,
            0,1,0,1,1,0,1,0,
            }, 1, 1, 1);

        static Image HideMonsterImage = Display.CreateImage(8, 8, new byte[] {
            0,0,0,0,0,0,0,0,
            0,0,0,0,0,0,0,0,
            0,0,0,0,0,0,0,0,
            0,0,0,0,0,0,0,0,
            0,0,0,0,0,0,0,0,
            0,0,0,0,0,0,0,0,
            0,0,0,0,0,0,0,0,
            0,0,0,0,0,0,0,0,
            }, 1, 1, 1);

        static Sprite Ship = new Sprite(ShipImage, 56, 58);
        static Sprite Bullet = new Sprite(BulletImage, 0, -10);
        static Sprite Monster = new Sprite(MonsterImage, 0, 0);

        static void ProcessShip()
        {
            Ship.Clear();
            if (BtnA.In() > 0)
            {
                Ship.X -= 5;
            }
            if (BtnB.In() > 0)
            {
                Ship.X += 5;
            }
            if (Ship.X < 5) Ship.X = 5;
            if (Ship.X > 110 - Ship.Width) Ship.X = 110 - Ship.Width;
            Ship.Draw();
        }

        static void ProcessBullet()
        {
            if (Bullet.Y > -5)
            {
                Bullet.Clear();
                Bullet.Y -= 5;
                Bullet.Draw();
                speaker.Out(3000 - Bullet.Y * 30);
            }
            else
            {
                if ((BtnA.In() > 0) && (BtnB.In() > 0))
                {
                    Bullet.X = Ship.X + Ship.Width / 2;
                    Bullet.Y = 55;
                }
            }
        }

        static void ProcessMonster()
        {
            Monster.Clear();
            Monster.X += MonsterDelta;

            if (Monster.X < 5 || Monster.X > 105 - Monster.Width)
            {
                MonsterDelta *= -1; // Reverse
                Monster.Y += 10;
            }
            Monster.Draw();
        }

        static void ProcessCollisions()
        {
            // Check the bullet
            if (Bullet.X >= Monster.X && Bullet.X <= Monster.X + Monster.Width &&
                        Bullet.Y >= Monster.Y && Bullet.Y <= Monster.Y + Monster.Height)
            {
                for (int i = 0; i < 3; i++)
                {
                    for (int frequency = 1000; frequency < 6000; frequency += 500)
                    {
                        speaker.Out(frequency);
                    }
                    Display.Image(HideMonsterImage, Monster.X, Monster.Y); Display.Show();
                    Display.Image(MonsterReverseImage, Monster.X, Monster.Y); Display.Show();
                    Controller.Wait(0.5);
                    Display.Image(HideMonsterImage, Monster.X, Monster.Y); Display.Show();
                    Display.Image(MonsterImage, Monster.X, Monster.Y); Display.Show();
                    Controller.Wait(0.1);
                }
                Bullet.Clear();
                Monster.Clear();
                Monster.Y = 0;
                Bullet.Y = -5;
                score += 1;
            }

            // Did we lose?
            if (Monster.Y > 40)
            {
                if (Ship.X > Monster.X && Ship.X < Monster.X + Ship.Width)
                {
                    for (int frequency = 3000; frequency > 500; frequency -= 250)
                    {
                        speaker.Out(frequency);
                    }
                    Monster.Clear();
                    Monster.Y = 0;
                    ShipNumber -= 1;
                }
            }
        }

        static void Main()
        {
            var WScreen = 128;
            var HScreen = 64;

            // Welcome
            Display.Text("Space Invasion", 20, 0);
            Display.Text("A : Left", 0, 12);
            Display.Text("b : Right", 0, 22);
            Display.Text("Fire : A & B", 0, 32);
            Display.Text("Start : A", 0, 55);
            Display.Image(MonsterImage, 110, 32);
            Display.Image(BulletImage, 113, 46);
            Display.Image(ShipImage, 110, 55);
            Display.Show();

            // Waiting for Start
            Display.Clear();
            while (BtnA.In() == 0) ;
            Display.Line(112, 0, 112, HScreen);
            Display.Image(ShipImage, WScreen - Ship.Width, 5);
            Display.Image(ShipImage, WScreen - Ship.Width, 15);
            Display.Image(ShipImage, WScreen - Ship.Width, 25);
            Display.Text(score.ToString(), WScreen - Ship.Width, 50);
            Ship.Draw();
            Display.Show();

            while (true)
            {
                if (ShipNumber == 0)
                {
                    Display.Clear();
                    Display.TextEx("Game Over", 5, 10, 2, 2);
                    Display.Text("Score = " + score.ToString(), 60, 55);
                    Display.Show();
                    while (true)
                    {
                        Controller.Wait(0.01); 
                    };
                }
                else
                {
                    ProcessCollisions();
                    Display.Clear();
                    Display.Line(112, 0, 112, HScreen);
                    switch (ShipNumber)
                    {
                        case 1:
                            Display.Image(ShipImage, WScreen - Ship.Width, 5);
                            break;
                        case 2:
                            Display.Image(ShipImage, WScreen - Ship.Width, 5);
                            Display.Image(ShipImage, WScreen - Ship.Width, 15);
                            break;
                        case 3:
                            Display.Image(ShipImage, WScreen - Ship.Width, 5);
                            Display.Image(ShipImage, WScreen - Ship.Width, 15);
                            Display.Image(ShipImage, WScreen - Ship.Width, 25);
                            break;
                        default:
                            break;
                    }
                    Display.Text(score.ToString(), WScreen - Ship.Width, 50);
                    ProcessShip();
                    ProcessBullet();
                    ProcessMonster();
                    Display.Show();
                    Controller.Wait(0.005);
                }
            }
        }
    }
}
